package com.riclage.boardview;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Build;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.ViewGroup;

import com.example.mangoswordsearch.R;


/**
 * Used by Julio on 30/3/2021
 * Created by Ricardo on 27/01/2016.
 */

//Contains BoardView contents which are the number of rows/cols it has and the TileSize as well as the boardWidth and boardHeight
    //This is where the coloring of the Board happens
public abstract class BoardView extends ViewGroup {

    private int numRows = 10; //Board Rows (Does Not work?)
    private int numCols = 10; //Board Columns (Does Not Work?)

    private int maxTileSize;
    private int tileSize;

    private Paint boardPaint;

    private int boardWidth, boardHeight;

    public BoardView(Context context) {
        super(context);
        init(null);
    }

    public BoardView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(attrs);
    }

    public BoardView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(attrs);
    }

    @SuppressWarnings("unused")
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public BoardView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(attrs);
    }

    public abstract void clearBoard();

    @SuppressLint("ResourceAsColor")
    private void init(AttributeSet attrs) {
        if (attrs != null) {
            TypedArray attrsArray = getContext().obtainStyledAttributes(attrs, R.styleable.boardview);
            maxTileSize = attrsArray.getDimensionPixelOffset(R.styleable.boardview_bvMaxTileSize, R.dimen.max_tile_size); //Makes tiles bigger or smaller
            attrsArray.recycle();
        }

        boardPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        boardPaint.setStyle(Paint.Style.STROKE); //Style of the Board Drawing
        boardPaint.setColor(Color.BLACK); //Changes outline of Board

        setWillNotDraw(false);
    }

    public void setBoardSize(int nRows, int nCols) { //sets the board size using the number of rows and columns
        numRows = nRows;
        numCols = nCols;
        invalidate();
    }

    //Changed NumRows and NumCols to not be static
    public int getNumRows() {
        return numRows;
    }

    public int getNumCols() {
        return numCols;
    }

    public int getTileSize() {
        return tileSize;
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) { //Sets the TileSize and Width/Height of the Board using math
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        tileSize = Math.round(Math.min(maxTileSize, Math.min((float)getMeasuredWidth() / numCols, (float)getMeasuredHeight() / numRows)));

        boardWidth = Math.min(tileSize * numCols, getMeasuredWidth());
        boardHeight = Math.min(tileSize * numRows, getMeasuredHeight());
        setMeasuredDimension(boardWidth, boardHeight);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        //Disable parent scrolling when the user is interacting with the board
        switch (event.getActionMasked()) {
            case MotionEvent.ACTION_DOWN:
                getParent().requestDisallowInterceptTouchEvent(true);
                break;
            case MotionEvent.ACTION_UP:
                getParent().requestDisallowInterceptTouchEvent(false);
                break;
        }
        return super.onTouchEvent(event);
    }

    @Override
    protected void dispatchDraw(Canvas canvas) {
        super.dispatchDraw(canvas);

        for (int i = 0; i <= numRows; i++) {
            //For custom grid sizes, y can't be equal the board size or the line drawn won't show
            int y = Math.min(boardHeight - 1, i * tileSize);
            canvas.drawLine(0, y, boardWidth, y, boardPaint);
        }

        for (int i = 0; i <= numCols; i++) {
            //For custom grid sizes, x can't be equal the board size or the line drawn won't show
            int x = Math.min(boardWidth - 1, i * tileSize);
            canvas.drawLine(x, 0, x, boardHeight, boardPaint);
        }
    }
}
